from trac.core import *
from trac.search import ISearchSource, shorten_result
from trac.versioncontrol.api import Node
from trac.perm import IPermissionRequestor
from trac.util import Markup, escape
from trac.mimeview.api import Mimeview
import re
import posixpath
import os
import string
from fnmatch import fnmatch

class SupoSEPlugin(Component):
    """ Search the source repository. """
    implements(ISearchSource, IPermissionRequestor)

    def _get_filters(self):
        includes = [glob for glob in self.env.config.get('supose',
                   'include', '').split(os.path.pathsep) if glob]
        excludes = [glob for glob in self.env.config.get('supose',
                   'exclude', '').split(os.path.pathsep) if glob]
        supose = [glob for glob in self.env.config.get('supose',
                   'supose', '').split(os.path.pathsep) if glob]
        index = [glob for glob in self.env.config.get('supose',
                   'index', '').split(os.path.pathsep) if glob]
        return (includes, excludes, supose, index)

    # IPermissionRequestor methods
    def get_permission_actions(self):
        yield 'REPO_SEARCH'

    # ISearchSource methods
    def get_search_filters(self, req):
        if req.perm.has_permission('REPO_SEARCH'):
            yield ('repo', 'Repository', 0)

    def get_search_results(self, req, query, filters):
        if 'repo' not in filters:
            return
        # if not isinstance(query, list):
            # query = query.split()
        query = [q.lower() for q in query]
        db = self.env.get_db_cnx()
        # include, excludes, supose, index = self._get_filters()
        to_unicode = Mimeview(self.env).to_unicode
        
        if not req.perm.has_permission('REPO_SEARCH'):
            return
        supose = self.env.config.get('supose', 'supose' )
        index = self.env.config.get('supose', 'index' )
        indexedrev = self.env.config.get('supose', 'indexedrev' )
        repo = self.env.get_repository(authname=req.authname)
        
        # Index with SupoSE
        youngest = int( repo.youngest_rev )
        
        base = re.search("(svn:.*:)(.*:.*)", repo.get_base() )
        base = base.group(2)
        base = "file:///" +  base
        
        if not indexedrev:
            # First Scan
            first_scan_cmd = supose + " scan --url "
            first_scan_cmd += base
            first_scan_cmd += " --create --index "
            first_scan_cmd += index
            scan_res = os.popen( first_scan_cmd ).read()
            indexedrev = youngest
            self.env.config.set('supose', 'indexedrev',
                       youngest)
            self.env.config.save()
        indexedrev = int( indexedrev )
        if indexedrev < youngest:
            new_index_cmd = supose + " scan --url "
            new_index_cmd += base
            new_index_cmd += " --fromrev " + str( indexedrev )
            new_index_cmd += " --index " + index
            scan_res = os.popen( new_index_cmd ).read()
            # raise Exception( new_index_cmd )
            self.env.config.set('supose', 'indexedrev',
                       youngest)
            self.env.config.save()
            
        # SupoSE search
        supose_cmd = str( supose )
        supose_cmd += " search --fields revision " 
        supose_cmd += "filename path contents --index "
        supose_cmd += index + " --query \""
       
        for q in query:
            q_cmd = supose_cmd + q + "\""
            repo_res = os.popen( q_cmd ).read()
            
            repo_reg = "(.*[\d]+:[ ]+REVISION:)([\d]+)"
            repo_reg += "( +FILENAME:)(.+)"
            repo_reg += "( +PATH:)(.+)"
            repo_reg += "( +CONTENTS:)"

            hits = re.split( repo_reg, repo_res ) 
            
            spit_len = 8
            rng = range( 1, len(hits), spit_len )
            for r in rng:
                rev = hits[r+1]
                filename = hits[r+3]
                path = hits[r+5]
                contents = to_unicode( hits[r+7] )
                change = repo.get_changeset(rev)
                # raise Exception( self.env.href.browser(path + filename ))
                yield ( self.env.href.browser( path + filename ) + "?rev=" + rev ,
                       path + filename + "@" + rev, change.date, 
                       change.author, 
                       shorten_result( contents, query ) )
        